@@ -43,12 +43,20 @@ eel_uri_is_starred (const gchar *uri)
     return g_str_has_prefix (uri, "starred:");
 }
 
+/* It also matches trashed folders inside Trash,
+ * use `eel_uri_is_trash_root` if that's not desirable. */
 gboolean
 eel_uri_is_trash (const char *uri)
 {
     return g_str_has_prefix (uri, "trash:");
 }
 
+gboolean
+eel_uri_is_trash_root (const char *uri)
+{
+    return g_strcmp0 (uri, "trash:///") == 0;
+}
+
 gboolean
 eel_uri_is_recent (const char *uri)
 {
@@ -99,6 +107,15 @@ eel_uri_is_in_xdg_dirs (const gchar *uri)
     return has_prefix;
 }
 
+/**
+ * eel_filename_get_extension_offset:
+ * @filename: a null-terminated string representing the basename of a file, with
+ *            or without extension.
+ *
+ * Returns: (nullable) (transfer none): A pointer to the substring containing
+ *                                      the dot and extension, or %NULL if there
+ *                                      is no extension.
+ */
 char *
 eel_filename_get_extension_offset (const char *filename)
 {
