@@ -276,13 +276,16 @@ icon_get_data_binder (NautilusCanvasIcon *icon,
     return TRUE;
 }
 
+typedef gboolean (*CanvasContainerEachFunc)(NautilusCanvasIcon *,
+                                            gpointer);
+
 /* Iterate over each selected icon in a NautilusCanvasContainer,
  * calling each_function on each.
  */
 static void
 nautilus_canvas_container_each_selected_icon (NautilusCanvasContainer *container,
-                                              gboolean (*each_function)(NautilusCanvasIcon *, gpointer),
-                                              gpointer data)
+                                              CanvasContainerEachFunc  each_function,
+                                              gpointer                 data)
 {
     GList *p;
     NautilusCanvasIcon *icon;
@@ -1492,8 +1495,6 @@ check_hover_timer (NautilusCanvasContainer *container,
                    const char              *uri)
 {
     NautilusCanvasDndInfo *dnd_info;
-    GtkSettings *settings;
-    guint timeout;
 
     dnd_info = container->details->dnd_info;
 
@@ -1504,19 +1505,12 @@ check_hover_timer (NautilusCanvasContainer *container,
 
     remove_hover_timer (dnd_info);
 
-    settings = gtk_widget_get_settings (GTK_WIDGET (container));
-    g_object_get (settings, "gtk-timeout-expand", &timeout, NULL);
-
-    g_free (dnd_info->target_uri);
-    dnd_info->target_uri = NULL;
+    g_clear_pointer (&dnd_info->target_uri, g_free);
 
     if (uri != NULL)
     {
         dnd_info->target_uri = g_strdup (uri);
-        dnd_info->hover_id =
-            gdk_threads_add_timeout (timeout,
-                                     hover_timer,
-                                     container);
+        dnd_info->hover_id = g_timeout_add (HOVER_TIMEOUT, hover_timer, container);
     }
 }
 
