@@ -116,6 +116,8 @@ struct NautilusCanvasItemDetails
 
     guint is_visible : 1;
 
+    guint is_thumbnail : 1;
+
     /* Cached PangoLayouts. Only used if the icon is visible */
     PangoLayout *editable_text_layout;
     PangoLayout *additional_text_layout;
@@ -472,7 +474,8 @@ get_scaled_icon_size (NautilusCanvasItem *item,
 
 void
 nautilus_canvas_item_set_image (NautilusCanvasItem *item,
-                                GdkPixbuf          *image)
+                                GdkPixbuf          *image,
+                                gboolean            is_thumbnail)
 {
     NautilusCanvasItemDetails *details;
 
@@ -488,6 +491,7 @@ nautilus_canvas_item_set_image (NautilusCanvasItem *item,
     if (image != NULL)
     {
         g_object_ref (image);
+        details->is_thumbnail = is_thumbnail;
     }
     if (details->pixbuf != NULL)
     {
@@ -1362,12 +1366,25 @@ nautilus_canvas_item_draw (EelCanvasItem  *item,
     icon_rect = canvas_item->details->icon_rect;
     temp_surface = map_surface (canvas_item);
 
+    if (details->is_thumbnail)
+    {
+        /* Draw box shadow before drawing the thumbnail. */
+        gtk_style_context_save (context);
+        gtk_style_context_add_class (context, "thumbnail");
+        gtk_render_background (context, cr,
+                               icon_rect.x0,
+                               icon_rect.y0,
+                               icon_rect.x1 - icon_rect.x0,
+                               icon_rect.y1 - icon_rect.y0);
+        gtk_style_context_restore (context);
+    }
     gtk_render_icon_surface (context, cr,
                              temp_surface,
                              icon_rect.x0, icon_rect.y0);
     cairo_surface_destroy (temp_surface);
 
     /* Draw the label text. */
+    gtk_style_context_add_class (context, "label-text");
     draw_label_text (canvas_item, cr, icon_rect);
 
     gtk_style_context_restore (context);
