@@ -30,6 +30,8 @@
 #include <eel/eel-string.h>
 #include <eel/eel-stock-dialogs.h>
 
+#include "nautilus-ui-utilities.h"
+
 #define DEBUG_FLAG NAUTILUS_DEBUG_DIRECTORY_VIEW
 #include "nautilus-debug.h"
 
@@ -110,7 +112,10 @@ nautilus_report_error_loading_directory (NautilusFile *file,
         message = g_strdup (error->message);
     }
 
-    eel_show_error_dialog (_("This location could not be displayed."), message, parent_window);
+    show_dialog (_("This location could not be displayed."),
+                 message,
+                 parent_window,
+                 GTK_MESSAGE_ERROR);
 }
 
 void
@@ -162,7 +167,7 @@ nautilus_report_error_setting_group (NautilusFile *file,
     }
 
 
-    eel_show_error_dialog (_("The group could not be changed."), message, parent_window);
+    show_dialog (_("The group could not be changed."), message, parent_window, GTK_MESSAGE_ERROR);
 }
 
 void
@@ -186,7 +191,7 @@ nautilus_report_error_setting_owner (NautilusFile *file,
     message = g_strdup_printf (_("Sorry, could not change the owner of “%s”: %s"),
                                truncated_name, truncated_error_message);
 
-    eel_show_error_dialog (_("The owner could not be changed."), message, parent_window);
+    show_dialog (_("The owner could not be changed."), message, parent_window, GTK_MESSAGE_ERROR);
 }
 
 void
@@ -210,7 +215,10 @@ nautilus_report_error_setting_permissions (NautilusFile *file,
     message = g_strdup_printf (_("Sorry, could not change the permissions of “%s”: %s"),
                                truncated_name, truncated_error_message);
 
-    eel_show_error_dialog (_("The permissions could not be changed."), message, parent_window);
+    show_dialog (_("The permissions could not be changed."),
+                 message,
+                 parent_window,
+                 GTK_MESSAGE_ERROR);
 }
 
 typedef struct _NautilusRenameData
@@ -311,7 +319,7 @@ nautilus_report_error_renaming_file (NautilusFile *file,
                                    truncated_error_message);
     }
 
-    eel_show_error_dialog (_("The item could not be renamed."), message, parent_window);
+    show_dialog (_("The item could not be renamed."), message, parent_window, GTK_MESSAGE_ERROR);
 }
 
 static void
