@@ -86,11 +86,7 @@ nautilus_files_view_handle_netscape_url_drop (NautilusFilesView *view,
 
     f = g_file_new_for_uri (url);
 
-    /* We don't support GDK_ACTION_ASK or GDK_ACTION_PRIVATE
-     * and we don't support combinations either. */
-    if ((action != GDK_ACTION_DEFAULT) &&
-        (action != GDK_ACTION_COPY) &&
-        (action != GDK_ACTION_MOVE))
+    if ((action != GDK_ACTION_COPY) && (action != GDK_ACTION_MOVE))
     {
         show_dialog (_("Drag and drop is not supported."),
                      _("An invalid drag type was used."),
@@ -147,10 +143,7 @@ nautilus_files_view_handle_uri_list_drop (NautilusFilesView *view,
         }
     }
 
-    /* We don't support GDK_ACTION_ASK or GDK_ACTION_PRIVATE
-     * and we don't support combinations either. */
-    if ((action != GDK_ACTION_DEFAULT) &&
-        (action != GDK_ACTION_COPY) &&
+    if ((action != GDK_ACTION_COPY) &&
         (action != GDK_ACTION_MOVE) &&
         (action != GDK_ACTION_LINK))
     {
@@ -408,7 +401,7 @@ nautilus_files_view_handle_hover (NautilusFilesView *view,
         !(current_location != NULL && g_file_equal (location, current_location)))
     {
         nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
-                                                 location, NAUTILUS_WINDOW_OPEN_FLAG_DONT_MAKE_ACTIVE,
+                                                 location, NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE,
                                                  NULL, NULL, slot);
     }
     g_object_unref (location);
