@@ -41,7 +41,7 @@ GSettings *gtk_filechooser_preferences;
 GSettings *gnome_lockdown_preferences;
 GSettings *gnome_background_preferences;
 GSettings *gnome_interface_preferences;
-GSettings *tracker_preferences;
+GSettings *gnome_privacy_preferences;
 
 void
 nautilus_global_preferences_init (void)
@@ -66,5 +66,5 @@ nautilus_global_preferences_init (void)
     gnome_lockdown_preferences = g_settings_new ("org.gnome.desktop.lockdown");
     gnome_background_preferences = g_settings_new ("org.gnome.desktop.background");
     gnome_interface_preferences = g_settings_new ("org.gnome.desktop.interface");
-    tracker_preferences = g_settings_new ("org.freedesktop.Tracker3.Miner.Files");
+    gnome_privacy_preferences = g_settings_new ("org.gnome.desktop.privacy");
 }
