@@ -22,8 +22,7 @@
             Anders Carlsson <andersca@gnu.org>
 */
 
-#ifndef NAUTILUS_LIST_VIEW_H
-#define NAUTILUS_LIST_VIEW_H
+#pragma once
 
 #include "nautilus-files-view.h"
 
@@ -38,6 +37,4 @@ struct _NautilusListView
 	NautilusListViewDetails *details;
 };
 
-NautilusFilesView * nautilus_list_view_new (NautilusWindowSlot *slot);
-
-#endif /* NAUTILUS_LIST_VIEW_H */
+NautilusFilesView * nautilus_list_view_new (NautilusWindowSlot *slot);
\ No newline at end of file
