@@ -212,8 +212,12 @@ action_pathbar_properties (GSimpleAction *action,
 static void
 on_adjustment_changed (GtkAdjustment *adjustment)
 {
+    gboolean rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
+
     /* Automatically scroll to the end, to keep the current folder visible */
-    gtk_adjustment_set_value (adjustment, gtk_adjustment_get_upper (adjustment));
+    gtk_adjustment_set_value (adjustment, (rtl ?
+                                           gtk_adjustment_get_lower (adjustment) :
+                                           gtk_adjustment_get_upper (adjustment)));
 }
 
 static gboolean
@@ -841,9 +845,9 @@ nautilus_path_bar_update_button_appearance (ButtonData *button_data,
          *
          * Due to variable width fonts, labels can be shorter than the space
          * that would be reserved by setting a minimum amount of characters.
-         * Compensate for this with a tolerance of +1 characters.
+         * Compensate for this with a tolerance of +50% characters.
          */
-        if (g_utf8_strlen (dir_name, -1) > min_chars + 1)
+        if (g_utf8_strlen (dir_name, -1) > min_chars * 1.5)
         {
             gtk_label_set_width_chars (GTK_LABEL (button_data->label), min_chars);
             gtk_label_set_ellipsize (GTK_LABEL (button_data->label), PANGO_ELLIPSIZE_MIDDLE);
