@@ -49,12 +49,6 @@ struct _NautilusTagManager
 
 G_DEFINE_TYPE (NautilusTagManager, nautilus_tag_manager, G_TYPE_OBJECT);
 
-typedef enum
-{
-    GET_STARRED_FILES,
-    GET_IDS_FOR_URLS
-} OperationType;
-
 typedef struct
 {
     NautilusTagManager *tag_manager;
@@ -88,7 +82,8 @@ static guint signals[LAST_SIGNAL];
 /* Limit to 10MB output from Tracker -- surely, nobody has over a million starred files. */
 #define TRACKER2_MAX_IMPORT_BYTES 10 * 1024 * 1024
 
-static const gchar *tracker2_migration_stamp (void)
+static gchar *
+tracker2_migration_stamp (void)
 {
     return g_build_filename (g_get_user_data_dir (), "nautilus", "tracker2-migration-complete", NULL);
 }
@@ -178,43 +173,6 @@ on_update_callback (GObject      *object,
     g_free (data);
 }
 
-static gboolean
-get_query_status (TrackerSparqlCursor *cursor,
-                  GAsyncResult        *result,
-                  OperationType        op_type,
-                  gpointer             user_data)
-{
-    gboolean success;
-    GTask *task;
-    g_autoptr (GError) error = NULL;
-
-    task = user_data;
-
-    success = tracker_sparql_cursor_next_finish (cursor, result, &error);
-
-    if (!success)
-    {
-        if (error)
-        {
-            g_warning ("Error on getting all tags cursor callback: %s", error->message);
-        }
-
-        g_clear_object (&cursor);
-
-        if (error == NULL ||
-            (error != NULL && error->code == G_IO_ERROR_CANCELLED))
-        {
-            if (op_type == GET_IDS_FOR_URLS)
-            {
-                g_task_return_pointer (task, g_task_get_task_data (task), NULL);
-                g_object_unref (task);
-            }
-        }
-    }
-
-    return success;
-}
-
 /**
  * nautilus_tag_manager_get_starred_files:
  * @self: The tag manager singleton
@@ -250,6 +208,7 @@ on_get_starred_files_cursor_callback (GObject      *object,
                                       gpointer      user_data)
 {
     TrackerSparqlCursor *cursor;
+    g_autoptr (GError) error = NULL;
     const gchar *url;
     gboolean success;
     NautilusTagManager *self;
@@ -260,9 +219,16 @@ on_get_starred_files_cursor_callback (GObject      *object,
 
     self = NAUTILUS_TAG_MANAGER (user_data);
 
-    success = get_query_status (cursor, result, GET_STARRED_FILES, NULL);
+    success = tracker_sparql_cursor_next_finish (cursor, result, &error);
+
     if (!success)
     {
+        if (error != NULL)
+        {
+            g_warning ("Error on getting all tags cursor callback: %s", error->message);
+        }
+
+        g_clear_object (&cursor);
         return;
     }
 
@@ -573,6 +539,7 @@ nautilus_tag_manager_finalize (GObject *object)
     g_clear_object (&self->query_starred_files);
 
     g_hash_table_destroy (self->starred_file_uris);
+    g_clear_object (&self->home);
 
     G_OBJECT_CLASS (nautilus_tag_manager_parent_class)->finalize (object);
 }
@@ -598,6 +565,16 @@ nautilus_tag_manager_class_init (NautilusTagManagerClass *klass)
                                              G_TYPE_POINTER);
 }
 
+/**
+ * nautilus_tag_manager_get:
+ *
+ * Gets a reference to the tag manager.
+ *
+ * If used to initialize a struct field, make sure to release on finalization.
+ * If used to initialize a local variable, make sure to use g_autoptr().
+ *
+ * Returns: (transfer full): the #NautilusTagManager singleton object.
+ */
 NautilusTagManager *
 nautilus_tag_manager_get (void)
 {
@@ -736,7 +713,7 @@ update_moved_uris_callback (GObject      *object,
     else
     {
         g_autolist (NautilusFile) updated_files = NULL;
-        g_autoptr (NautilusTagManager) tag_manager = NULL;
+        g_autoptr (NautilusTagManager) tag_manager = nautilus_tag_manager_get ();
 
         for (guint i = 0; i < new_uris->len; i++)
         {
@@ -745,7 +722,6 @@ update_moved_uris_callback (GObject      *object,
             updated_files = g_list_prepend (updated_files, nautilus_file_get_by_uri (new_uri));
         }
 
-        tag_manager = nautilus_tag_manager_get ();
         g_signal_emit_by_name (tag_manager, "starred-changed", updated_files);
     }
 }
@@ -857,7 +833,7 @@ process_tracker2_data_cb (GObject      *source_object,
                           gpointer      user_data)
 {
     NautilusTagManager *self = NAUTILUS_TAG_MANAGER (source_object);
-    const gchar *path = tracker2_migration_stamp ();
+    g_autofree gchar *path = tracker2_migration_stamp ();
     g_autoptr (GError) error = NULL;
 
     tracker_sparql_connection_update_finish (self->db, res, &error);
@@ -1004,7 +980,9 @@ export_tracker2_data (NautilusTagManager *self)
 void
 nautilus_tag_manager_maybe_migrate_tracker2_data (NautilusTagManager *self)
 {
-    if (g_file_test (tracker2_migration_stamp (), G_FILE_TEST_EXISTS))
+    g_autofree gchar *path = tracker2_migration_stamp ();
+
+    if (g_file_test (path, G_FILE_TEST_EXISTS))
     {
         DEBUG ("Tracker 2 migration: already completed.");
     }
