@@ -40,7 +40,7 @@
 #include "nautilus-toolbar-menu-sections.h"
 #include "nautilus-ui-utilities.h"
 #include "nautilus-window.h"
-#include "nautilus-container-max-width.h"
+#include "nautilus-gtk4-helpers.h"
 
 #define OPERATION_MINIMUM_TIME 2 /*s */
 #define NEEDS_ATTENTION_ANIMATION_TIMEOUT 2000 /*ms */
@@ -50,7 +50,6 @@
 #define ANIMATION_Y_GROW 30
 
 /* Just design, context at https://gitlab.gnome.org/GNOME/nautilus/issues/548#note_274131 */
-#define SWITCHER_MAX_WIDTH 840
 
 typedef enum
 {
@@ -69,8 +68,6 @@ struct _NautilusToolbar
     GtkWidget *location_entry_container;
     GtkWidget *search_container;
     GtkWidget *toolbar_switcher;
-    GtkWidget *toolbar_switcher_container;
-    NautilusContainerMaxWidth *toolbar_switcher_container_max_width;
     GtkWidget *path_bar;
     GtkWidget *location_entry;
 
@@ -82,27 +79,27 @@ struct _NautilusToolbar
     guint operations_button_attention_timeout_id;
 
     GtkWidget *operations_button;
-    GtkWidget *view_button;
-    GtkWidget *view_menu_zoom_section;
-    GtkWidget *view_menu_undo_redo_section;
-    GtkWidget *view_menu_extended_section;
-    GtkWidget *undo_button;
-    GtkWidget *redo_button;
-    GtkWidget *view_toggle_button;
-    GtkWidget *view_toggle_icon;
-
-    GtkWidget *app_menu;
-
     GtkWidget *operations_popover;
-    GtkWidget *operations_container;
+    GtkWidget *operations_list;
+    GListStore *progress_infos_model;
     GtkWidget *operations_revealer;
     GtkWidget *operations_icon;
 
+    GtkWidget *view_toggle_button;
+    GtkWidget *view_toggle_icon;
+    GtkWidget *view_button;
+    GMenuModel *view_menu;
+
+    GtkWidget *app_button;
+    GMenuModel *undo_redo_section;
+
     GtkWidget *forward_button;
+    GtkWidget *forward_menu;
     GtkGesture *forward_button_longpress_gesture;
     GtkGesture *forward_button_multi_press_gesture;
 
     GtkWidget *back_button;
+    GtkWidget *back_menu;
     GtkGesture *back_button_longpress_gesture;
     GtkGesture *back_button_multi_press_gesture;
 
@@ -160,42 +157,14 @@ toolbar_update_appearance (NautilusToolbar *self)
     }
 }
 
-static void
-activate_back_or_forward_menu_item (GtkMenuItem        *menu_item,
-                                    NautilusWindowSlot *window_slot,
-                                    gboolean            back)
-{
-    int index;
-
-    g_assert (GTK_IS_MENU_ITEM (menu_item));
-
-    index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item), "user_data"));
-
-    nautilus_window_slot_back_or_forward (window_slot, back, index);
-}
-
-static void
-activate_back_menu_item_callback (GtkMenuItem     *menu_item,
-                                  NautilusToolbar *self)
-{
-    activate_back_or_forward_menu_item (menu_item, self->window_slot, TRUE);
-}
-
-static void
-activate_forward_menu_item_callback (GtkMenuItem     *menu_item,
-                                     NautilusToolbar *self)
-{
-    activate_back_or_forward_menu_item (menu_item, self->window_slot, FALSE);
-}
-
 static void
 fill_menu (NautilusToolbar *self,
-           GtkWidget       *menu,
+           GMenu           *menu,
            gboolean         back)
 {
-    GtkWidget *menu_item;
-    int index;
+    guint index;
     GList *list;
+    const gchar *name;
 
     list = back ? nautilus_window_slot_get_back_history (self->window_slot) :
            nautilus_window_slot_get_forward_history (self->window_slot);
@@ -203,16 +172,15 @@ fill_menu (NautilusToolbar *self,
     index = 0;
     while (list != NULL)
     {
-        menu_item = nautilus_bookmark_menu_item_new (NAUTILUS_BOOKMARK (list->data));
-        g_object_set_data (G_OBJECT (menu_item), "user_data", GINT_TO_POINTER (index));
-        gtk_widget_show (GTK_WIDGET (menu_item));
-        g_signal_connect_object (menu_item, "activate",
-                                 back
-                                 ? G_CALLBACK (activate_back_menu_item_callback)
-                                 : G_CALLBACK (activate_forward_menu_item_callback),
-                                 self, 0);
+        g_autoptr (GMenuItem) item = NULL;
+
+        name = nautilus_bookmark_get_name (NAUTILUS_BOOKMARK (list->data));
+        item = g_menu_item_new (name, NULL);
+        g_menu_item_set_action_and_target (item,
+                                           back ? "win.back-n" : "win.forward-n",
+                                           "u", index);
+        g_menu_append_item (menu, item);
 
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
         list = g_list_next (list);
         ++index;
     }
@@ -220,13 +188,13 @@ fill_menu (NautilusToolbar *self,
 
 static void
 show_menu (NautilusToolbar *self,
-           GtkWidget       *widget,
-           const GdkEvent  *event)
+           GtkWidget       *widget)
 {
-    GtkWidget *menu;
+    g_autoptr (GMenu) menu = NULL;
     NautilusNavigationDirection direction;
+    GtkPopover *popover;
 
-    menu = gtk_menu_new ();
+    menu = g_menu_new ();
 
     direction = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget),
                                                      "nav-direction"));
@@ -236,12 +204,14 @@ show_menu (NautilusToolbar *self,
         case NAUTILUS_NAVIGATION_DIRECTION_FORWARD:
         {
             fill_menu (self, menu, FALSE);
+            popover = GTK_POPOVER (self->forward_menu);
         }
         break;
 
         case NAUTILUS_NAVIGATION_DIRECTION_BACK:
         {
             fill_menu (self, menu, TRUE);
+            popover = GTK_POPOVER (self->back_menu);
         }
         break;
 
@@ -252,11 +222,8 @@ show_menu (NautilusToolbar *self,
         break;
     }
 
-    gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (self->window), NULL);
-    gtk_menu_popup_at_widget (GTK_MENU (menu), widget,
-                              GDK_GRAVITY_SOUTH_WEST,
-                              GDK_GRAVITY_NORTH_WEST,
-                              event);
+    gtk_popover_bind_model (popover, G_MENU_MODEL (menu), NULL);
+    gtk_popover_popup (popover);
 }
 
 static void
@@ -268,15 +235,11 @@ navigation_button_press_cb (GtkGestureMultiPress *gesture,
 {
     NautilusToolbar *self;
     GtkWidget *widget;
-    GdkEventSequence *sequence;
-    const GdkEvent *event;
 
     self = NAUTILUS_TOOLBAR (user_data);
     widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (gesture));
-    sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
-    event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
 
-    show_menu (self, widget, event);
+    show_menu (self, widget);
 }
 
 static void
@@ -286,13 +249,8 @@ back_button_longpress_cb (GtkGestureLongPress *gesture,
                           gpointer             user_data)
 {
     NautilusToolbar *self = user_data;
-    GdkEventSequence *sequence;
-    const GdkEvent *event;
 
-    sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
-    event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
-
-    show_menu (self, self->back_button, event);
+    show_menu (self, self->back_button);
 }
 
 static void
@@ -302,13 +260,8 @@ forward_button_longpress_cb (GtkGestureLongPress *gesture,
                              gpointer             user_data)
 {
     NautilusToolbar *self = user_data;
-    GdkEventSequence *sequence;
-    const GdkEvent *event;
 
-    sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
-    event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
-
-    show_menu (self, self->forward_button, event);
+    show_menu (self, self->forward_button);
 }
 
 static gboolean
@@ -520,14 +473,10 @@ update_operations (NautilusToolbar *self)
 {
     GList *progress_infos;
     GList *l;
-    GtkWidget *progress;
     gboolean should_show_progress_button = FALSE;
 
-    gtk_container_foreach (GTK_CONTAINER (self->operations_container),
-                           (GtkCallback) gtk_widget_destroy,
-                           NULL);
-
     disconnect_progress_infos (self);
+    g_list_store_remove_all (self->progress_infos_model);
 
     progress_infos = get_filtered_progress_infos (self);
     for (l = progress_infos; l != NULL; l = l->next)
@@ -541,10 +490,7 @@ update_operations (NautilusToolbar *self)
                                   G_CALLBACK (on_progress_info_cancelled), self);
         g_signal_connect_swapped (l->data, "progress-changed",
                                   G_CALLBACK (on_progress_info_progress_changed), self);
-        progress = nautilus_progress_info_widget_new (l->data);
-        gtk_box_pack_start (GTK_BOX (self->operations_container),
-                            progress,
-                            FALSE, FALSE, 0);
+        g_list_store_append (self->progress_infos_model, l->data);
     }
 
     g_list_free (progress_infos);
@@ -687,7 +633,7 @@ on_operations_icon_draw (GtkWidget       *widget,
     GtkStyleContext *style_context;
 
     style_context = gtk_widget_get_style_context (widget);
-    gtk_style_context_get_color (style_context, gtk_widget_get_state_flags (widget), &foreground);
+    gtk_style_context_get_color (style_context, gtk_style_context_get_state (style_context), &foreground);
     background = foreground;
     background.alpha *= 0.3;
 
@@ -778,28 +724,15 @@ on_progress_has_viewers_changed (NautilusProgressInfoManager *manager,
 }
 
 static void
-update_menu_item (GtkWidget       *menu_item,
-                  NautilusToolbar *self,
-                  const char      *action_name,
-                  gboolean         enabled,
-                  char            *label)
+update_action (NautilusToolbar *self,
+               const char      *action_name,
+               gboolean         enabled)
 {
     GAction *action;
-    GValue val = G_VALUE_INIT;
 
     /* Activate/deactivate */
     action = g_action_map_lookup_action (G_ACTION_MAP (self->window), action_name);
     g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
-
-    /* Set the text of the menu item. Can't use gtk_button_set_label here as
-     * we need to set the text property, not the label. There's no equivalent
-     * gtk_model_button_set_text function (refer to #766083 for discussion
-     * on adding a set_text function)
-     */
-    g_value_init (&val, G_TYPE_STRING);
-    g_value_set_string (&val, label);
-    g_object_set_property (G_OBJECT (menu_item), "text", &val);
-    g_value_unset (&val);
 }
 
 static void
@@ -814,6 +747,8 @@ undo_manager_changed (NautilusToolbar *self)
     g_autofree gchar *undo_description = NULL;
     g_autofree gchar *redo_description = NULL;
     gboolean is_undo;
+    g_autoptr (GMenu) updated_section = g_menu_new ();
+    g_autoptr (GMenuItem) menu_item = NULL;
 
     /* Look up the last action from the undo manager, and get the text that
      * describes it, e.g. "Undo Create Folder"/"Redo Create Folder"
@@ -841,14 +776,21 @@ undo_manager_changed (NautilusToolbar *self)
         g_free (undo_label);
         undo_label = g_strdup (_("_Undo"));
     }
-    update_menu_item (self->undo_button, self, "undo", undo_active, undo_label);
+    g_set_object (&menu_item, g_menu_item_new (undo_label, "win.undo"));
+    g_menu_append_item (updated_section, menu_item);
+    update_action (self, "undo", undo_active);
 
     if (!redo_active || redo_label == NULL)
     {
         g_free (redo_label);
         redo_label = g_strdup (_("_Redo"));
     }
-    update_menu_item (self->redo_button, self, "redo", redo_active, redo_label);
+    g_set_object (&menu_item, g_menu_item_new (redo_label, "win.redo"));
+    g_menu_append_item (updated_section, menu_item);
+    update_action (self, "redo", redo_active);
+
+    nautilus_gmenu_set_from_model (G_MENU (self->undo_redo_section),
+                                   G_MENU_MODEL (updated_section));
 }
 
 static void
@@ -891,37 +833,35 @@ on_location_entry_focus_changed (GObject    *object,
     }
 }
 
+static GtkWidget *
+operations_list_create_widget (GObject  *item,
+                               gpointer  user_data)
+{
+    NautilusProgressInfo *info = NAUTILUS_PROGRESS_INFO (item);
+    GtkWidget *widget;
+
+    widget = nautilus_progress_info_widget_new (info);
+    gtk_widget_show_all (widget);
+
+    return widget;
+}
+
 static void
 nautilus_toolbar_constructed (GObject *object)
 {
-    g_autoptr (GtkBuilder) builder = NULL;
     NautilusToolbar *self = NAUTILUS_TOOLBAR (object);
 
-    builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/ui/nautilus-toolbar-switcher.ui");
-    self->toolbar_switcher = GTK_WIDGET (gtk_builder_get_object (builder, "toolbar_switcher"));
-    self->search_container = GTK_WIDGET (gtk_builder_get_object (builder, "search_container"));
-    self->path_bar_container = GTK_WIDGET (gtk_builder_get_object (builder, "path_bar_container"));
-    self->location_entry_container = GTK_WIDGET (gtk_builder_get_object (builder, "location_entry_container"));
-
-    self->toolbar_switcher_container_max_width = nautilus_container_max_width_new ();
-    nautilus_container_max_width_set_max_width (self->toolbar_switcher_container_max_width,
-                                                SWITCHER_MAX_WIDTH);
-    gtk_container_add (GTK_CONTAINER (self->toolbar_switcher_container_max_width),
-                       self->toolbar_switcher);
-    gtk_container_add (GTK_CONTAINER (self->toolbar_switcher_container),
-                       GTK_WIDGET (self->toolbar_switcher_container_max_width));
-
-    self->path_bar = g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL);
-    gtk_container_add (GTK_CONTAINER (self->path_bar_container),
-                       self->path_bar);
+    self->path_bar = GTK_WIDGET (g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL));
+    gtk_box_append (GTK_BOX (self->path_bar_container),
+                    self->path_bar);
 
     self->location_entry = nautilus_location_entry_new ();
-    gtk_container_add (GTK_CONTAINER (self->location_entry_container),
-                       self->location_entry);
+    gtk_box_append (GTK_BOX (self->location_entry_container),
+                    self->location_entry);
     self->location_entry_close_button = gtk_button_new_from_icon_name ("window-close-symbolic",
                                                                        GTK_ICON_SIZE_BUTTON);
-    gtk_container_add (GTK_CONTAINER (self->location_entry_container),
-                       self->location_entry_close_button);
+    gtk_box_append (GTK_BOX (self->location_entry_container),
+                    self->location_entry_close_button);
     g_signal_connect (self->location_entry_close_button, "clicked",
                       G_CALLBACK (on_location_entry_close), self);
 
@@ -931,6 +871,12 @@ nautilus_toolbar_constructed (GObject *object)
     g_signal_connect (self->progress_manager, "has-viewers-changed",
                       G_CALLBACK (on_progress_has_viewers_changed), self);
 
+    self->progress_infos_model = g_list_store_new (NAUTILUS_TYPE_PROGRESS_INFO);
+    gtk_list_box_bind_model (GTK_LIST_BOX (self->operations_list),
+                             G_LIST_MODEL (self->progress_infos_model),
+                             (GtkListBoxCreateWidgetFunc) operations_list_create_widget,
+                             NULL,
+                             NULL);
     update_operations (self);
 
     self->back_button_longpress_gesture = gtk_gesture_long_press_new (self->back_button);
@@ -968,6 +914,10 @@ nautilus_toolbar_constructed (GObject *object)
     g_signal_connect (self->location_entry, "notify::has-focus",
                       G_CALLBACK (on_location_entry_focus_changed), self);
 
+    /* Setting a max width on one entry to effectively set a max expansion for
+     * the whole title widget. */
+    gtk_entry_set_max_width_chars (GTK_ENTRY (self->location_entry), 88);
+
     gtk_widget_show_all (GTK_WIDGET (self));
     toolbar_update_appearance (self);
 }
@@ -1164,6 +1114,7 @@ nautilus_toolbar_finalize (GObject *obj)
     unschedule_operations_start (self);
     unschedule_operations_button_attention_style (self);
 
+    g_clear_object (&self->progress_infos_model);
     g_signal_handlers_disconnect_by_data (self->progress_manager, self);
     g_clear_object (&self->progress_manager);
 
@@ -1227,21 +1178,22 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, operations_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, operations_icon);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, operations_popover);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, operations_container);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, operations_list);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, operations_revealer);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_button);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_menu);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_toggle_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_toggle_icon);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, app_menu);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, app_button);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, undo_redo_section);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, back_button);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, back_menu);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, forward_button);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, toolbar_switcher_container);
-
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_menu_zoom_section);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_menu_undo_redo_section);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_menu_extended_section);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, undo_button);
-    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, redo_button);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, forward_menu);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, toolbar_switcher);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_container);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, path_bar_container);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, location_entry_container);
 
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, search_button);
 
@@ -1282,17 +1234,13 @@ nautilus_toolbar_set_show_location_entry (NautilusToolbar *self,
 }
 
 static void
-container_remove_all_children (GtkContainer *container)
+box_remove_all_children (GtkBox *box)
 {
-    GList *children;
-    GList *child;
-
-    children = gtk_container_get_children (container);
-    for (child = children; child != NULL; child = g_list_next (child))
+    GtkWidget *child;
+    while ((child = gtk_widget_get_first_child (GTK_WIDGET (box))) != NULL)
     {
-        gtk_container_remove (container, GTK_WIDGET (child->data));
+        gtk_box_remove (GTK_BOX (box), child);
     }
-    g_list_free (children);
 }
 
 static void
@@ -1325,34 +1273,29 @@ on_slot_toolbar_menu_sections_changed (NautilusToolbar    *self,
                                        NautilusWindowSlot *slot)
 {
     NautilusToolbarMenuSections *new_sections;
-
-    container_remove_all_children (GTK_CONTAINER (self->view_menu_zoom_section));
-    container_remove_all_children (GTK_CONTAINER (self->view_menu_extended_section));
+    g_autoptr (GMenuItem) zoom_item = NULL;
+    g_autoptr (GMenuItem) sort_item = NULL;
 
     new_sections = nautilus_window_slot_get_toolbar_menu_sections (slot);
+
+    gtk_widget_set_sensitive (self->view_button, (new_sections != NULL));
     if (new_sections == NULL)
     {
         return;
     }
 
-    gtk_widget_set_visible (self->view_menu_undo_redo_section,
-                            new_sections->supports_undo_redo);
+    /* Let's assume that zoom and sort sections are the first and second items
+     * in view_menu, as per nautilus-toolbar.ui. */
 
-    if (new_sections->zoom_section != NULL)
-    {
-        gtk_box_pack_start (GTK_BOX (self->view_menu_zoom_section),
-                            new_sections->zoom_section, FALSE, FALSE, 0);
-    }
+    zoom_item = g_menu_item_new_from_model (self->view_menu, 0);
+    g_menu_remove (G_MENU (self->view_menu), 0);
+    g_menu_item_set_section (zoom_item, new_sections->zoom_section);
+    g_menu_insert_item (G_MENU (self->view_menu), 0, zoom_item);
 
-    if (new_sections->extended_section != NULL)
-    {
-        gtk_box_pack_start (GTK_BOX (self->view_menu_extended_section),
-                            new_sections->extended_section, FALSE, FALSE, 0);
-    }
-
-    gtk_widget_set_sensitive (self->view_button, (new_sections->extended_section != NULL ||
-                                                  new_sections->zoom_section != NULL ||
-                                                  new_sections->supports_undo_redo));
+    sort_item = g_menu_item_new_from_model (self->view_menu, 1);
+    g_menu_remove (G_MENU (self->view_menu), 1);
+    g_menu_item_set_section (sort_item, new_sections->sort_section);
+    g_menu_insert_item (G_MENU (self->view_menu), 1, sort_item);
 }
 
 
@@ -1457,17 +1400,12 @@ nautilus_toolbar_set_window_slot_real (NautilusToolbar    *self,
                                   G_CALLBACK (toolbar_update_appearance), self);
     }
 
-    children = gtk_container_get_children (GTK_CONTAINER (self->search_container));
-    if (children != NULL)
-    {
-        gtk_container_remove (GTK_CONTAINER (self->search_container),
-                              children->data);
-    }
+    box_remove_all_children (GTK_BOX (self->search_container));
 
     if (self->window_slot != NULL)
     {
-        gtk_container_add (GTK_CONTAINER (self->search_container),
-                           GTK_WIDGET (nautilus_window_slot_get_query_editor (self->window_slot)));
+        gtk_box_append (GTK_BOX (self->search_container),
+                        GTK_WIDGET (nautilus_window_slot_get_query_editor (self->window_slot)));
     }
 
     toolbar_update_appearance (self);
@@ -1505,9 +1443,14 @@ nautilus_toolbar_set_window_slot (NautilusToolbar    *self,
 gboolean
 nautilus_toolbar_is_menu_visible (NautilusToolbar *self)
 {
+    GtkWidget *menu;
+
     g_return_val_if_fail (NAUTILUS_IS_TOOLBAR (self), FALSE);
 
-    return gtk_widget_is_visible (self->app_menu);
+    menu = GTK_WIDGET (gtk_menu_button_get_popover (GTK_MENU_BUTTON (self->app_button)));
+    g_return_val_if_fail (menu != NULL, FALSE);
+
+    return gtk_widget_is_visible (menu);
 }
 
 gboolean
