@@ -32,11 +32,9 @@
 
 typedef struct
 {
-    gboolean waiting_for_data;
     gboolean have_data;
     gboolean have_valid_data;
 
-    gboolean is_hover;
     gboolean drop_occurred;
 
     unsigned int info;
@@ -96,7 +94,7 @@ switch_location (NautilusDragSlotProxyInfo *drag_info)
 
     location = nautilus_file_get_location (drag_info->target_file);
     nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
-                                             location, NAUTILUS_WINDOW_OPEN_FLAG_DONT_MAKE_ACTIVE,
+                                             location, NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE,
                                              NULL, NAUTILUS_WINDOW (window), NULL);
     g_object_unref (location);
 }
@@ -124,21 +122,14 @@ static void
 slot_proxy_check_switch_location_timer (NautilusDragSlotProxyInfo *drag_info,
                                         GtkWidget                 *widget)
 {
-    GtkSettings *settings;
-    guint timeout;
-
     if (drag_info->switch_location_timer)
     {
         return;
     }
 
-    settings = gtk_widget_get_settings (widget);
-    g_object_get (settings, "gtk-timeout-expand", &timeout, NULL);
-
-    drag_info->switch_location_timer =
-        gdk_threads_add_timeout (timeout,
-                                 slot_proxy_switch_location_timer,
-                                 drag_info);
+    drag_info->switch_location_timer = g_timeout_add (HOVER_TIMEOUT,
+                                                      slot_proxy_switch_location_timer,
+                                                      drag_info);
 }
 
 static void
@@ -171,8 +162,6 @@ slot_proxy_drag_motion (GtkWidget      *widget,
 
     drag_info = user_data;
 
-    drag_info->is_hover = TRUE;
-
     action = 0;
     valid_text_drag = FALSE;
     valid_xds_drag = FALSE;
@@ -185,7 +174,7 @@ slot_proxy_drag_motion (GtkWidget      *widget,
     window = gtk_widget_get_toplevel (widget);
     g_assert (NAUTILUS_IS_WINDOW (window));
 
-    if (!drag_info->have_data && !drag_info->waiting_for_data)
+    if (!drag_info->have_data)
     {
         target = gtk_drag_dest_find_target (widget, context, NULL);
 
@@ -194,7 +183,6 @@ slot_proxy_drag_motion (GtkWidget      *widget,
             goto out;
         }
 
-        drag_info->waiting_for_data = TRUE;
         gtk_drag_get_data (widget, context, target, time);
     }
 
@@ -338,7 +326,6 @@ slot_proxy_drag_leave (GtkWidget      *widget,
 
     drag_info = user_data;
 
-    drag_info->is_hover = FALSE;
     gtk_drag_unhighlight (widget);
     drag_info_clear (drag_info);
 }
@@ -359,12 +346,8 @@ slot_proxy_drag_drop (GtkWidget      *widget,
 
     drag_info->drop_occurred = TRUE;
 
-    if (!drag_info->waiting_for_data)
-    {
-        target = gtk_drag_dest_find_target (widget, context, NULL);
-        drag_info->waiting_for_data = TRUE;
-        gtk_drag_get_data (widget, context, target, time);
-    }
+    target = gtk_drag_dest_find_target (widget, context, NULL);
+    gtk_drag_get_data (widget, context, target, time);
 
     return TRUE;
 }
@@ -476,28 +459,16 @@ slot_proxy_drag_data_received (GtkWidget        *widget,
     drag_info = user_data;
 
     g_assert (!drag_info->have_data);
-    drag_info->waiting_for_data = FALSE;
-
-    if (!drag_info->is_hover && !drag_info->drop_occurred)
-    {
-        /* Ignore data arriving after ::drag-leave, except if followed by
-         * ::drag-drop. */
-        return;
-    }
-
-    if (gtk_selection_data_get_length (data) < 0)
-    {
-        /* Data retrieval failed. */
-        if (drag_info->drop_occurred)
-        {
-            gtk_drag_finish (context, FALSE, FALSE, time);
-        }
-        return;
-    }
 
     drag_info->have_data = TRUE;
     drag_info->info = info;
 
+    if (gtk_selection_data_get_length (data) < 0)
+    {
+        drag_info->have_valid_data = FALSE;
+        return;
+    }
+
     if (info == NAUTILUS_ICON_DND_GNOME_ICON_LIST)
     {
         drag_info->data.selection_list = nautilus_drag_build_selection_list (data);
