@@ -52,10 +52,10 @@ NautilusWindow * nautilus_window_slot_get_window           (NautilusWindowSlot *
 void             nautilus_window_slot_set_window           (NautilusWindowSlot *slot,
 							    NautilusWindow     *window);
 
-void nautilus_window_slot_open_location_full              (NautilusWindowSlot      *slot,
-                                                           GFile                   *location,
-                                                           NautilusWindowOpenFlags  flags,
-                                                           GList                   *new_selection);
+void nautilus_window_slot_open_location_full               (NautilusWindowSlot *slot,
+                                                            GFile              *location,
+                                                            NautilusOpenFlags   flags,
+                                                            GList              *new_selection);
 
 GFile * nautilus_window_slot_get_location		   (NautilusWindowSlot *slot);
 GFile * nautilus_window_slot_get_pending_location          (NautilusWindowSlot *slot);
@@ -73,8 +73,10 @@ void     nautilus_window_slot_stop_loading                 (NautilusWindowSlot *
 const gchar *nautilus_window_slot_get_title                (NautilusWindowSlot *slot);
 void         nautilus_window_slot_update_title		   (NautilusWindowSlot *slot);
 
-gboolean nautilus_window_slot_handle_event       	   (NautilusWindowSlot *slot,
-							    GdkEvent           *event);
+gboolean nautilus_window_slot_handle_event       	   (NautilusWindowSlot    *slot,
+							    GtkEventControllerKey *controller,
+							    guint                  keyval,
+							    GdkModifierType        state);
 
 void    nautilus_window_slot_queue_reload		   (NautilusWindowSlot *slot);
 
