@@ -69,6 +69,7 @@
 #include "nautilus-trash-monitor.h"
 #include "nautilus-ui-utilities.h"
 #include "nautilus-window-slot.h"
+#include "nautilus-gtk4-helpers.h"
 
 /* Forward and back buttons on the mouse */
 static gboolean mouse_extra_buttons = TRUE;
@@ -151,6 +152,8 @@ struct _NautilusWindow
 
     GtkGesture *multi_press_gesture;
     GtkGesture *notebook_multi_press_gesture;
+    GtkEventController *key_capture_controller;
+    GtkEventController *key_bubble_controller;
 };
 
 enum
@@ -164,27 +167,6 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (NautilusWindow, nautilus_window, HDY_TYPE_APPLICATION_WINDOW);
 
-static const struct
-{
-    unsigned int keyval;
-    const char *action;
-} extra_window_keybindings [] =
-{
-    /* Window actions */
-    { GDK_KEY_AddFavorite, "bookmark-current-location" },
-    { GDK_KEY_Favorites, "bookmarks" },
-    { GDK_KEY_Go, "enter-location" },
-    { GDK_KEY_HomePage, "go-home" },
-    { GDK_KEY_OpenURL, "enter-location" },
-    { GDK_KEY_Refresh, "reload" },
-    { GDK_KEY_Reload, "reload" },
-    { GDK_KEY_Search, "search" },
-    { GDK_KEY_Start, "go-home" },
-    { GDK_KEY_Stop, "stop" },
-    { GDK_KEY_Back, "back" },
-    { GDK_KEY_Forward, "forward" },
-};
-
 static const GtkPadActionEntry pad_actions[] =
 {
     { GTK_PAD_ACTION_BUTTON, 0, -1, N_("Parent folder"), "up" },
@@ -225,6 +207,20 @@ action_go_home (GSimpleAction *action,
     g_object_unref (home);
 }
 
+static void
+action_go_starred (GSimpleAction *action,
+                   GVariant      *state,
+                   gpointer       user_data)
+{
+    NautilusWindow *window;
+    g_autoptr (GFile) starred = NULL;
+
+    window = NAUTILUS_WINDOW (user_data);
+    starred = g_file_new_for_uri ("starred:///");
+
+    nautilus_window_open_location_full (window, starred, 0, NULL, NULL);
+}
+
 static void
 action_reload (GSimpleAction *action,
                GVariant      *state,
@@ -292,6 +288,26 @@ action_forward (GSimpleAction *action,
     nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data), FALSE, 0);
 }
 
+static void
+action_back_n (GSimpleAction *action,
+               GVariant      *parameter,
+               gpointer       user_data)
+{
+    nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data),
+                                     TRUE,
+                                     g_variant_get_uint32 (parameter));
+}
+
+static void
+action_forward_n (GSimpleAction *action,
+                  GVariant      *parameter,
+                  gpointer       user_data)
+{
+    nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data),
+                                     FALSE,
+                                     g_variant_get_uint32 (parameter));
+}
+
 static void
 action_bookmark_current_location (GSimpleAction *action,
                                   GVariant      *state,
@@ -444,6 +460,19 @@ action_toggle_state_view_button (GSimpleAction *action,
     g_variant_unref (current_state);
 }
 
+static void
+action_show_current_location_menu (GSimpleAction *action,
+                                   GVariant      *state,
+                                   gpointer       user_data)
+{
+    NautilusWindow *window = user_data;
+    GtkWidget *path_bar;
+
+    path_bar = nautilus_toolbar_get_path_bar (NAUTILUS_TOOLBAR (window->toolbar));
+
+    nautilus_path_bar_show_current_location_menu (NAUTILUS_PATH_BAR (path_bar));
+}
+
 static void
 on_location_changed (NautilusWindow *window)
 {
@@ -498,8 +527,8 @@ disconnect_slot (NautilusWindow     *window,
 }
 
 static NautilusWindowSlot *
-nautilus_window_create_and_init_slot (NautilusWindow          *window,
-                                      NautilusWindowOpenFlags  flags)
+nautilus_window_create_and_init_slot (NautilusWindow    *window,
+                                      NautilusOpenFlags  flags)
 {
     NautilusWindowSlot *slot;
 
@@ -510,9 +539,9 @@ nautilus_window_create_and_init_slot (NautilusWindow          *window,
 }
 
 void
-nautilus_window_initialize_slot (NautilusWindow          *window,
-                                 NautilusWindowSlot      *slot,
-                                 NautilusWindowOpenFlags  flags)
+nautilus_window_initialize_slot (NautilusWindow     *window,
+                                 NautilusWindowSlot *slot,
+                                 NautilusOpenFlags   flags)
 {
     g_assert (NAUTILUS_IS_WINDOW (window));
     g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
@@ -524,7 +553,7 @@ nautilus_window_initialize_slot (NautilusWindow          *window,
                                      window);
     nautilus_notebook_add_tab (NAUTILUS_NOTEBOOK (window->notebook),
                                slot,
-                               (flags & NAUTILUS_WINDOW_OPEN_SLOT_APPEND) != 0 ?
+                               (flags & NAUTILUS_OPEN_FLAG_SLOT_APPEND) != 0 ?
                                -1 :
                                gtk_notebook_get_current_page (GTK_NOTEBOOK (window->notebook)) + 1,
                                FALSE);
@@ -537,24 +566,24 @@ nautilus_window_initialize_slot (NautilusWindow          *window,
 }
 
 void
-nautilus_window_open_location_full (NautilusWindow          *window,
-                                    GFile                   *location,
-                                    NautilusWindowOpenFlags  flags,
-                                    GList                   *selection,
-                                    NautilusWindowSlot      *target_slot)
+nautilus_window_open_location_full (NautilusWindow     *window,
+                                    GFile              *location,
+                                    NautilusOpenFlags   flags,
+                                    GList              *selection,
+                                    NautilusWindowSlot *target_slot)
 {
     NautilusWindowSlot *active_slot;
     gboolean new_tab_at_end;
 
     /* Assert that we are not managing new windows */
-    g_assert (!(flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW));
+    g_assert (!(flags & NAUTILUS_OPEN_FLAG_NEW_WINDOW));
     /* if the flags say we want a new tab, open a slot in the current window */
-    if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0)
+    if ((flags & NAUTILUS_OPEN_FLAG_NEW_TAB) != 0)
     {
         new_tab_at_end = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_NEW_TAB_POSITION) == NAUTILUS_NEW_TAB_POSITION_END;
         if (new_tab_at_end)
         {
-            flags |= NAUTILUS_WINDOW_OPEN_SLOT_APPEND;
+            flags |= NAUTILUS_OPEN_FLAG_SLOT_APPEND;
         }
     }
 
@@ -564,14 +593,14 @@ nautilus_window_open_location_full (NautilusWindow          *window,
         target_slot = active_slot;
     }
 
-    if (target_slot == NULL || (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0)
+    if (target_slot == NULL || (flags & NAUTILUS_OPEN_FLAG_NEW_TAB) != 0)
     {
         target_slot = nautilus_window_create_and_init_slot (window, flags);
     }
 
     /* Make the opened location the one active if we weren't ask for the
      * oposite, since it's the most usual use case */
-    if (!(flags & NAUTILUS_WINDOW_OPEN_FLAG_DONT_MAKE_ACTIVE))
+    if (!(flags & NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE))
     {
         gtk_window_present (GTK_WINDOW (window));
         nautilus_window_set_active_slot (window, target_slot);
@@ -707,7 +736,7 @@ nautilus_window_new_tab (NautilusWindow *window)
         }
 
         nautilus_window_open_location_full (window, location,
-                                            NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB,
+                                            NAUTILUS_OPEN_FLAG_NEW_TAB,
                                             NULL, NULL);
         g_object_unref (location);
     }
@@ -809,12 +838,12 @@ save_sidebar_width_cb (gpointer user_data)
 
 /* side pane helpers */
 static void
-side_pane_size_allocate_callback (GtkWidget     *widget,
-                                  GtkAllocation *allocation,
-                                  gpointer       user_data)
+side_pane_notify_position_callback (GObject    *object,
+                                    GParamSpec *pspec,
+                                    gpointer    user_data)
 {
     NautilusWindow *window = user_data;
-
+    gint position;
 
     if (window->sidebar_width_handler_id != 0)
     {
@@ -822,10 +851,11 @@ side_pane_size_allocate_callback (GtkWidget     *widget,
         window->sidebar_width_handler_id = 0;
     }
 
-    if (allocation->width != window->side_pane_width &&
-        allocation->width > 1)
+    position = gtk_paned_get_position (GTK_PANED (window->content_paned));
+    if (position != window->side_pane_width &&
+        position > 1)
     {
-        window->side_pane_width = allocation->width;
+        window->side_pane_width = position;
 
         window->sidebar_width_handler_id =
             g_idle_add (save_sidebar_width_cb, window);
@@ -851,21 +881,21 @@ open_location_cb (NautilusWindow     *window,
                   GFile              *location,
                   GtkPlacesOpenFlags  open_flags)
 {
-    NautilusWindowOpenFlags flags;
+    NautilusOpenFlags flags;
     NautilusApplication *application;
 
     switch (open_flags)
     {
         case GTK_PLACES_OPEN_NEW_TAB:
         {
-            flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB |
-                    NAUTILUS_WINDOW_OPEN_FLAG_DONT_MAKE_ACTIVE;
+            flags = NAUTILUS_OPEN_FLAG_NEW_TAB |
+                    NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE;
         }
         break;
 
         case GTK_PLACES_OPEN_NEW_WINDOW:
         {
-            flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+            flags = NAUTILUS_OPEN_FLAG_NEW_WINDOW;
         }
         break;
 
@@ -1136,7 +1166,7 @@ action_restore_tab (GSimpleAction *action,
                     gpointer       user_data)
 {
     NautilusWindow *window = NAUTILUS_WINDOW (user_data);
-    NautilusWindowOpenFlags flags;
+    NautilusOpenFlags flags;
     g_autoptr (GFile) location = NULL;
     NautilusWindowSlot *slot;
     NautilusNavigationState *data;
@@ -1146,7 +1176,7 @@ action_restore_tab (GSimpleAction *action,
         return;
     }
 
-    flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB | NAUTILUS_WINDOW_OPEN_FLAG_DONT_MAKE_ACTIVE;
+    flags = NAUTILUS_OPEN_FLAG_NEW_TAB | NAUTILUS_OPEN_FLAG_DONT_MAKE_ACTIVE;
 
     data = g_queue_pop_head (window->tab_data_queue);
 
@@ -1207,7 +1237,7 @@ add_menu_separator (GtkWidget *menu)
     GtkWidget *separator;
 
     separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-    gtk_container_add (GTK_CONTAINER (menu), separator);
+    gtk_box_append (GTK_BOX (menu), separator);
     gtk_widget_show (separator);
 }
 
@@ -1237,7 +1267,7 @@ places_sidebar_populate_popup_cb (GtkPlacesSidebar *sidebar,
             gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_item),
                                             "win.empty-trash");
             g_object_set (menu_item, "text", _("Empty _Trash…"), NULL);
-            gtk_container_add (GTK_CONTAINER (menu), menu_item);
+            gtk_box_append (GTK_BOX (menu), menu_item);
             gtk_widget_show (menu_item);
 
             action = g_action_map_lookup_action (G_ACTION_MAP (window),
@@ -1256,7 +1286,7 @@ places_sidebar_populate_popup_cb (GtkPlacesSidebar *sidebar,
             gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_item),
                                             "win.properties");
             g_object_set (menu_item, "text", _("_Properties"), NULL);
-            gtk_container_add (GTK_CONTAINER (menu), menu_item);
+            gtk_box_append (GTK_BOX (menu), menu_item);
             gtk_widget_show (menu_item);
         }
     }
@@ -1272,7 +1302,7 @@ places_sidebar_populate_popup_cb (GtkPlacesSidebar *sidebar,
             {
                 window->selected_volume = g_object_ref (selected_volume);
             }
-            gtk_container_add (GTK_CONTAINER (menu), menu_item);
+            gtk_box_append (GTK_BOX (menu), menu_item);
             gtk_widget_show (menu_item);
 
             action = g_action_map_lookup_action (G_ACTION_MAP (window),
@@ -1288,9 +1318,9 @@ static void
 nautilus_window_set_up_sidebar (NautilusWindow *window)
 {
     setup_side_pane_width (window);
-    g_signal_connect (window->sidebar,
-                      "size-allocate",
-                      G_CALLBACK (side_pane_size_allocate_callback),
+    g_signal_connect (window->content_paned,
+                      "notify::position",
+                      G_CALLBACK (side_pane_notify_position_callback),
                       window);
 
     gtk_places_sidebar_set_open_flags (GTK_PLACES_SIDEBAR (window->places_sidebar),
@@ -1413,12 +1443,16 @@ nautilus_window_sync_location_widgets (NautilusWindow *window)
         nautilus_path_bar_set_path (NAUTILUS_PATH_BAR (path_bar), location);
     }
 
-    action = g_action_map_lookup_action (G_ACTION_MAP (window), "back");
     enabled = nautilus_window_slot_get_back_history (slot) != NULL;
+    action = g_action_map_lookup_action (G_ACTION_MAP (window), "back");
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
+    action = g_action_map_lookup_action (G_ACTION_MAP (window), "back-n");
     g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
 
-    action = g_action_map_lookup_action (G_ACTION_MAP (window), "forward");
     enabled = nautilus_window_slot_get_forward_history (slot) != NULL;
+    action = g_action_map_lookup_action (G_ACTION_MAP (window), "forward");
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
+    action = g_action_map_lookup_action (G_ACTION_MAP (window), "forward-n");
     g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enabled);
 
     nautilus_window_sync_bookmarks (window);
@@ -1587,7 +1621,7 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
             /* Don't pop up a notification if user canceled the operation or the focus
              * is not in the this window. This is an easy way to know from which window
              * was the delete operation made */
-            if (files != NULL && gtk_window_has_toplevel_focus (GTK_WINDOW (window)))
+            if (files != NULL && gtk_window_is_active (GTK_WINDOW (window)))
             {
                 popup_notification = TRUE;
                 label = in_app_notification_undo_deleted_get_label (undo_info);
@@ -1604,7 +1638,7 @@ nautilus_window_on_undo_changed (NautilusFileUndoManager *manager,
              * window. This is an easy way to know from which window was the
              * unstart operation made */
             if (eel_uri_is_starred (g_file_get_uri (location)) &&
-                gtk_window_has_toplevel_focus (GTK_WINDOW (window)) &&
+                gtk_window_is_active (GTK_WINDOW (window)) &&
                 !nautilus_file_undo_info_starred_is_starred (NAUTILUS_FILE_UNDO_INFO_STARRED (undo_info)))
             {
                 popup_notification = TRUE;
@@ -1683,7 +1717,7 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
     }
 
     current_location = nautilus_window_slot_get_location (window->active_slot);
-    if (gtk_window_has_toplevel_focus (GTK_WINDOW (window)))
+    if (gtk_window_is_active (GTK_WINDOW (window)))
     {
         remove_notifications (window);
         gtk_label_set_text (GTK_LABEL (window->notification_operation_label),
@@ -1715,11 +1749,19 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
 }
 
 static void
-path_bar_location_changed_callback (GtkWidget      *widget,
-                                    GFile          *location,
-                                    NautilusWindow *window)
+on_path_bar_open_location (NautilusWindow    *window,
+                           GFile             *location,
+                           NautilusOpenFlags  open_flags)
 {
-    nautilus_window_open_location_full (window, location, 0, NULL, NULL);
+    if (open_flags & NAUTILUS_OPEN_FLAG_NEW_WINDOW)
+    {
+        nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
+                                                 location, NAUTILUS_OPEN_FLAG_NEW_WINDOW, NULL, NULL, NULL);
+    }
+    else
+    {
+        nautilus_window_open_location_full (window, location, open_flags, NULL, NULL);
+    }
 }
 
 static void
@@ -1879,10 +1921,8 @@ setup_toolbar (NautilusWindow *window)
     /* connect to the pathbar signals */
     path_bar = nautilus_toolbar_get_path_bar (NAUTILUS_TOOLBAR (window->toolbar));
 
-    g_signal_connect_object (path_bar, "path-clicked",
-                             G_CALLBACK (path_bar_location_changed_callback), window, 0);
     g_signal_connect_swapped (path_bar, "open-location",
-                              G_CALLBACK (open_location_cb), window);
+                              G_CALLBACK (on_path_bar_open_location), window);
 
     /* connect to the location entry signals */
     location_entry = nautilus_toolbar_get_location_entry (NAUTILUS_TOOLBAR (window->toolbar));
@@ -2007,8 +2047,11 @@ const GActionEntry win_entries[] =
 {
     { "back", action_back },
     { "forward", action_forward },
+    { "back-n", action_back_n, "u" },
+    { "forward-n", action_forward_n, "u" },
     { "up", action_up },
     { "view-menu", action_toggle_state_view_button, NULL, "false", NULL },
+    { "current-location-menu", action_show_current_location_menu },
     { "reload", action_reload },
     { "stop", action_stop },
     { "new-tab", action_new_tab },
@@ -2019,6 +2062,7 @@ const GActionEntry win_entries[] =
     /* Only accesible by shorcuts */
     { "close-current-view", action_close_current_view },
     { "go-home", action_go_home },
+    { "go-starred", action_go_starred },
     { "tab-previous", action_tab_previous },
     { "tab-next", action_tab_next },
     { "tab-move-left", action_tab_move_left },
@@ -2041,53 +2085,39 @@ nautilus_window_initialize_actions (NautilusWindow *window)
     gchar detailed_action[80];
     gchar accel[80];
     gint i;
-    const gchar *reload_accels[] =
-    {
-        "F5",
-        "<ctrl>r",
-        NULL
-    };
-    const gchar *prompt_root_location_accels[] =
-    {
-        "slash",
-        "KP_Divide",
-        NULL
-    };
-    const gchar *prompt_home_location_accels[] =
-    {
-        "asciitilde",
-        "dead_tilde",
-        NULL
-    };
 
     g_action_map_add_action_entries (G_ACTION_MAP (window),
                                      win_entries, G_N_ELEMENTS (win_entries),
                                      window);
 
+#define ACCELS(...) ((const char *[]) { __VA_ARGS__, NULL })
+
     app = g_application_get_default ();
-    nautilus_application_set_accelerator (app, "win.back", "<alt>Left");
-    nautilus_application_set_accelerator (app, "win.forward", "<alt>Right");
-    nautilus_application_set_accelerator (app, "win.enter-location", "<control>l");
+    nautilus_application_set_accelerators (app, "win.back", ACCELS ("<alt>Left", "Back"));
+    nautilus_application_set_accelerators (app, "win.forward", ACCELS ("<alt>Right", "Forward"));
+    nautilus_application_set_accelerators (app, "win.enter-location", ACCELS ("<control>l", "Go", "OpenURL"));
     nautilus_application_set_accelerator (app, "win.new-tab", "<control>t");
     nautilus_application_set_accelerator (app, "win.close-current-view", "<control>w");
 
     /* Special case reload, since users are used to use two shortcuts instead of one */
-    nautilus_application_set_accelerators (app, "win.reload", reload_accels);
+    nautilus_application_set_accelerators (app, "win.reload", ACCELS ("F5", "<ctrl>r", "Refresh", "Reload"));
+    nautilus_application_set_accelerator (app, "win.stop", "Stop");
 
     nautilus_application_set_accelerator (app, "win.undo", "<control>z");
     nautilus_application_set_accelerator (app, "win.redo", "<shift><control>z");
     /* Only accesible by shorcuts */
-    nautilus_application_set_accelerator (app, "win.bookmark-current-location", "<control>d");
+    nautilus_application_set_accelerators (app, "win.bookmark-current-location", ACCELS ("<control>d", "AddFavorite"));
     nautilus_application_set_accelerator (app, "win.up", "<alt>Up");
-    nautilus_application_set_accelerator (app, "win.go-home", "<alt>Home");
+    nautilus_application_set_accelerators (app, "win.go-home", ACCELS ("<alt>Home", "HomePage", "Start"));
+    nautilus_application_set_accelerator (app, "win.go-starred", "Favorites");
     nautilus_application_set_accelerator (app, "win.tab-previous", "<control>Page_Up");
     nautilus_application_set_accelerator (app, "win.tab-next", "<control>Page_Down");
     nautilus_application_set_accelerator (app, "win.tab-move-left", "<shift><control>Page_Up");
     nautilus_application_set_accelerator (app, "win.tab-move-right", "<shift><control>Page_Down");
-    nautilus_application_set_accelerators (app, "win.prompt-root-location", prompt_root_location_accels);
+    nautilus_application_set_accelerators (app, "win.prompt-root-location", ACCELS ("slash", "KP_Divide"));
     /* Support keyboard layouts which have a dead tilde key but not a tilde key. */
-    nautilus_application_set_accelerators (app, "win.prompt-home-location", prompt_home_location_accels);
-    nautilus_application_set_accelerator (app, "win.view-menu", "F10");
+    nautilus_application_set_accelerators (app, "win.prompt-home-location", ACCELS ("asciitilde", "dead_tilde"));
+    nautilus_application_set_accelerator (app, "win.current-location-menu", "F10");
     nautilus_application_set_accelerator (app, "win.restore-tab", "<shift><control>t");
 
     /* Alt+N for the first 9 tabs */
@@ -2098,6 +2128,8 @@ nautilus_window_initialize_actions (NautilusWindow *window)
         nautilus_application_set_accelerator (app, detailed_action, accel);
     }
 
+#undef ACCELS
+
     action = g_action_map_lookup_action (G_ACTION_MAP (app), "show-hide-sidebar");
     state = g_action_get_state (action);
     if (g_variant_get_boolean (state))
@@ -2182,14 +2214,14 @@ destroy_slots_foreach (gpointer data,
 }
 
 static void
-nautilus_window_destroy (GtkWidget *object)
+nautilus_window_dispose (GObject *object)
 {
     NautilusWindow *window;
-    NautilusApplication *application;
+    GtkApplication *application;
     GList *slots_copy;
 
     window = NAUTILUS_WINDOW (object);
-    application = NAUTILUS_APPLICATION (gtk_window_get_application (GTK_WINDOW (window)));
+    application = gtk_window_get_application (GTK_WINDOW (window));
 
     DEBUG ("Destroying window");
 
@@ -2209,24 +2241,21 @@ nautilus_window_destroy (GtkWidget *object)
 
     g_clear_weak_pointer (&window->active_slot);
 
-    g_clear_signal_handler (&window->bookmarks_id, nautilus_application_get_bookmarks (application));
+    if (application != NULL)
+    {
+        g_clear_signal_handler (&window->bookmarks_id,
+                                nautilus_application_get_bookmarks (NAUTILUS_APPLICATION (application)));
+    }
 
     g_clear_handle_id (&window->in_app_notification_undo_timeout_id, g_source_remove);
 
     nautilus_window_unexport_handle (window);
 
-    GTK_WIDGET_CLASS (nautilus_window_parent_class)->destroy (object);
-}
-
-static void
-nautilus_window_dispose (GObject *object)
-{
-    NautilusWindow *window;
-
-    window = NAUTILUS_WINDOW (object);
-
     g_clear_object (&window->notebook_multi_press_gesture);
 
+    g_clear_object (&window->key_capture_controller);
+    g_clear_object (&window->key_bubble_controller);
+
     G_OBJECT_CLASS (nautilus_window_parent_class)->dispose (object);
 }
 
@@ -2384,59 +2413,46 @@ nautilus_window_realize (GtkWidget *widget)
 }
 
 static gboolean
-nautilus_window_key_press_event (GtkWidget   *widget,
-                                 GdkEventKey *event)
+nautilus_window_key_capture (GtkEventControllerKey *controller,
+                             unsigned int           keyval,
+                             unsigned int           keycode,
+                             GdkModifierType        state,
+                             gpointer               user_data)
 {
-    NautilusWindow *window;
-    guint keyval;
+    GtkWidget *widget;
     GtkWidget *focus_widget;
 
-    window = NAUTILUS_WINDOW (widget);
-
-    if (G_UNLIKELY (!gdk_event_get_keyval ((GdkEvent *) event, &keyval)))
-    {
-        g_return_val_if_reached (GDK_EVENT_PROPAGATE);
-    }
-
-    focus_widget = gtk_window_get_focus (GTK_WINDOW (window));
+    widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (controller));
+    focus_widget = gtk_window_get_focus (GTK_WINDOW (widget));
     if (focus_widget != NULL && GTK_IS_EDITABLE (focus_widget))
     {
         /* if we have input focus on a GtkEditable (e.g. a GtkEntry), forward
-         * the event to it before activating accelerator bindings too.
+         * the event to it before activating accelerators. This allows, e.g.,
+         * typing a tilde without activating the prompt-home-location action.
          */
-        if (gtk_window_propagate_key_event (GTK_WINDOW (window),
-                                            (GdkEventKey *) event))
+        if (gtk_event_controller_key_forward (controller, focus_widget))
         {
             return GDK_EVENT_STOP;
         }
     }
 
-    for (int i = 0; i < G_N_ELEMENTS (extra_window_keybindings); i++)
-    {
-        if (extra_window_keybindings[i].keyval == keyval)
-        {
-            GAction *action;
+    return GDK_EVENT_PROPAGATE;
+}
 
-            action = g_action_map_lookup_action (G_ACTION_MAP (window), extra_window_keybindings[i].action);
-
-            g_assert (action != NULL);
-            if (g_action_get_enabled (action))
-            {
-                g_action_activate (action, NULL);
-                return GDK_EVENT_STOP;
-            }
-
-            break;
-        }
-    }
-
-    if (GTK_WIDGET_CLASS (nautilus_window_parent_class)->key_press_event (widget, event))
-    {
-        return GDK_EVENT_STOP;
-    }
+static gboolean
+nautilus_window_key_bubble (GtkEventControllerKey *controller,
+                            unsigned int           keyval,
+                            unsigned int           keycode,
+                            GdkModifierType        state,
+                            gpointer               user_data)
+{
+    GtkWidget *widget;
+    NautilusWindow *window;
 
+    widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (controller));
+    window = NAUTILUS_WINDOW (widget);
     if (window->active_slot != NULL &&
-        nautilus_window_slot_handle_event (window->active_slot, (GdkEvent *) event))
+        nautilus_window_slot_handle_event (window->active_slot, controller, keyval, state))
     {
         return GDK_EVENT_STOP;
     }
@@ -2732,6 +2748,20 @@ nautilus_window_init (NautilusWindow *window)
                                                 GTK_PHASE_CAPTURE);
     gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (window->notebook_multi_press_gesture),
                                    GDK_BUTTON_SECONDARY);
+
+    window->key_capture_controller = gtk_event_controller_key_new (GTK_WIDGET (window));
+    gtk_event_controller_set_propagation_phase (window->key_capture_controller,
+                                                GTK_PHASE_CAPTURE);
+    g_signal_connect (window->key_capture_controller,
+                      "key-pressed", G_CALLBACK (nautilus_window_key_capture),
+                      NULL);
+
+    window->key_bubble_controller = gtk_event_controller_key_new (GTK_WIDGET (window));
+    gtk_event_controller_set_propagation_phase (window->key_bubble_controller,
+                                                GTK_PHASE_BUBBLE);
+    g_signal_connect (window->key_bubble_controller,
+                      "key-pressed", G_CALLBACK (nautilus_window_key_bubble),
+                      NULL);
 }
 
 static void
@@ -2744,10 +2774,8 @@ nautilus_window_class_init (NautilusWindowClass *class)
     oclass->finalize = nautilus_window_finalize;
     oclass->constructed = nautilus_window_constructed;
 
-    wclass->destroy = nautilus_window_destroy;
     wclass->show = nautilus_window_show;
     wclass->realize = nautilus_window_realize;
-    wclass->key_press_event = nautilus_window_key_press_event;
     wclass->delete_event = nautilus_window_delete_event;
     wclass->grab_focus = nautilus_window_grab_focus;
 
